@@ -24,12 +24,15 @@ except ImportError:
 import shutil
 import socket
 import sys
+
 from .firefox_binary import FirefoxBinary
-from .service import Service
+from .remote_connection import FirefoxRemoteConnection
 from selenium.webdriver.common.desired_capabilities import DesiredCapabilities
 from selenium.webdriver.firefox.extension_connection import ExtensionConnection
 from selenium.webdriver.firefox.firefox_profile import FirefoxProfile
 from selenium.webdriver.remote.webdriver import WebDriver as RemoteWebDriver
+from .service import Service
+from .options import Options
 
 
 class WebDriver(RemoteWebDriver):
@@ -38,43 +41,63 @@ class WebDriver(RemoteWebDriver):
     NATIVE_EVENTS_ALLOWED = sys.platform != "darwin"
 
     def __init__(self, firefox_profile=None, firefox_binary=None, timeout=30,
-                 capabilities=None, proxy=None, executable_path='wires'):
-
-        self.binary = firefox_binary
+                 capabilities=None, proxy=None, executable_path="wires", firefox_options=None):
         self.profile = firefox_profile
+        self.binary = firefox_binary
+        if firefox_options is None:
 
-        if self.profile is None:
-            self.profile = FirefoxProfile()
+            if self.profile is None:
+                self.profile = FirefoxProfile()
 
-        self.profile.native_events_enabled = (
-            self.NATIVE_EVENTS_ALLOWED and self.profile.native_events_enabled)
+            self.profile.native_events_enabled = (
+                self.NATIVE_EVENTS_ALLOWED and self.profile.native_events_enabled)
 
-        if capabilities is None:
-            capabilities = DesiredCapabilities.FIREFOX
+            if capabilities is None:
+                capabilities = DesiredCapabilities.FIREFOX
+
+            if self.binary is None:
+                self.binary = capabilities.get("binary") or FirefoxBinary()
+
+            firefox_options = Options()
+            firefox_options.binary_location = self.binary if isinstance(self.binary, basestring) else self.binary._get_firefox_start_cmd()
+            firefox_options.profile = self.profile
+        else:
+            if capabilities is None:
+                capabilities = firefox_options.to_capabilities()
+            else:
+                capabilities.update(firefox_options.to_capabilities())
+
+        # marionette
+        if capabilities.get("marionette"):
+            self.binary = firefox_options.binary_location
+            if isinstance(firefox_options.binary_location, FirefoxBinary):
+                self.binary = firefox_options.binary_location._get_firefox_start_cmd()
 
-        if "marionette" in capabilities and capabilities['marionette'] is True:
-            # Let's use Marionette! WOOOOHOOOOO!
-            if "binary" in capabilities:
-                self.binary = capabilities["binary"]
             self.service = Service(executable_path, firefox_binary=self.binary)
             self.service.start()
 
-            RemoteWebDriver.__init__(self,
-                command_executor=self.service.service_url,
+            executor = FirefoxRemoteConnection(
+                remote_server_addr=self.service.service_url)
+            RemoteWebDriver.__init__(
+                self,
+                command_executor=executor,
                 desired_capabilities=capabilities,
                 keep_alive=True)
-
         else:
             # Oh well... sometimes the old way is the best way.
-            if self.binary is None:
-                self.binary = FirefoxBinary()
-
             if proxy is not None:
                 proxy.add_to_capabilities(capabilities)
 
+            if self.binary is None:
+                self.binary = firefox_options.binary_location or FirefoxBinary()
+
+            if self.profile is None:
+                self.profile = firefox_options.profile or FirefoxProfile()
+
+            executor = ExtensionConnection("127.0.0.1", self.profile,
+                                           self.binary, timeout)
             RemoteWebDriver.__init__(self,
-            command_executor=ExtensionConnection("127.0.0.1", self.profile,
-            self.binary, timeout),
+                command_executor=executor,
             desired_capabilities=capabilities,
             keep_alive=True)
 
@@ -89,7 +112,7 @@ class WebDriver(RemoteWebDriver):
             # Happens if Firefox shutsdown before we've read the response from
             # the socket.
             pass
-        if "marionette" in self.capabilities and self.capabilities['marionette'] is True:
+        if "specificationLevel" in self.capabilities:
             self.service.stop()
         else:
             self.binary.kill()
@@ -103,3 +126,6 @@ class WebDriver(RemoteWebDriver):
     @property
     def firefox_profile(self):
         return self.profile
+
+    def set_context(self, context):
+        self.execute("SET_CONTEXT", {"context": context})
